use crate::vm::types::DmType;
use crate::vm::PlatformCallFrame;
use crate::vm::DmObject;
use DmType::*;

pub struct DmString {
    data: String,
}

pub fn dm_print(frame: PlatformCallFrame) -> Result<Option<(u64, DmType)>, String> {
    if frame.args.len() != 1 {
        return Err(String::from("std::core::print requires one argument."));
    }
    match frame.arg_types[0] {
        Int => {
            print!("{}", frame.args[0] as i32);
        }
        Long => {
            print!("{}", frame.args[0] as i64);
        }
        Double => {
            print!("{}", frame.args[0] as f64);
        }
        Pointer => unsafe {
            let dm_object = Box::from_raw(frame.args[0] as *mut DmObject);
            if let Some(&ref to_string_method) = dm_object
                .object_type
                .methods
                .get(&String::from("to_string"))
            {
                let call_result = frame
                        .vm
                        .call(&to_string_method, [frame.args[0]], [frame.arg_types[0]]);
                if call_result.is_err() {
                    return Err(call_result.unwrap_err());
                }
                let (dm_string_address, _) = call_result?;
                // TODO: check that it's actually a DmString
                let dm_string = Box::from_raw(dm_string_address as *mut DmString);
                print!("{}", dm_string.data);
            } else {
                print!("{}@{:?}", dm_object.object_type.name, dm_object.data);
            }
        },
        Unit => {
            print!("Unit")
        },
        Exception => {
            print!("Exception")
        }
    }
    Ok(None)
}
